# This file contains common functions for the tomcat hooks.ec

REMOTE_SERVICE=${JUJU_REMOTE_UNIT%/*}
# $JUJU_REMOTE_UNIT variable holds the name of the unit which reported to have
# -joined, -changed, or -departed
SERVICE=${JUJU_UNIT_NAME%/*}
# $JUJU_UNIT_NAME variable holds the name of the unit.

add_file_row () {
  # The first argument is the file.
  local file=$1
  # Shift the arguments by one.
  shift
  if [ -e "${file}" ]; then
    # Use Extended grep with the invert flag (-v) to find all lines that 
    # do not start with the unit name (memcached/0).
    egrep -v "^$1"$'(\t|$)' "${file}" > "${file}.new" || true
    mv "${file}.new" "${file}"
  fi
  echo_delimited $'\t' "$@" >> "${file}"
}

check_checksum () {
  # Check the md5sum of file $1 and compare it against $2 exit on mismatch.
  if [ `md5sum "$1" | cut -d " " -f 1` != $2 ]; then
    juju-log --log-level CRITICAL "Checksum of $1 failed"
    exit 1
  fi
}

close_port () {
  juju-log "Closing port $1 for ${VERSION}"
  # Ask juju to close the port number.
  close-port $1
}

create_relation_id () {
  # Create a file for each relation in /etc/tomcatX/<relation_name>.relation
  echo "${REMOTE_SERVICE}" > /etc/${VERSION}/${JUJU_RELATION_ID}.relation
  # $JUJU_RELATION_ID variable holds the opaque relation identifier used to
  # distingush between multiple relations with the same name.
}

echo_delimited () {
  local d=$1
  printf %s "$2"
  # Shift the arguments by 2.
  shift 2
  local arg
  for arg; do
    printf %s%s "${d}" "${arg}"
  done
  echo
}

escape_name () {
  # Replace % $ and / characters in the name and print the new name.
  local name=${1//%/%25}
  name=${name//\$/%24}
  name=${name//\//%2F}
  printf %s "${name}"
}

escape_unit_name () {
  echo ${1//\//-}
}

open_port () {
  juju-log "Opening port $1 for ${VERSION}"
  # Ask Juju to open the port for this unit.
  open-port $1
}

remove_config () {
  # Get the path of the .d directory where the $1 file is built.
  if dotdee -d "$1" 2> /dev/null; then
    dotdee --undo "$1"
    rm -rf "/etc/dotdee$1" "/etc/dotdee$1.d"
  fi
}

remove_file_row () {
  if [ -e "$1" ]; then
    # Use Extended grep with reverse mach (-v) to find all lines that do not 
    # start with the unit name (memcached/0).
    if egrep -v "^$2"$'(\t|$)' "$1" > "$1.new"; then
      mv "$1.new" "$1"
    else
      rm -f "$1" "$1.new"
    fi
  fi
}

remove_relation_id () {
  # Delete the /etc/tomcatX/<relation_identifier>.relation file.
  rm -f /etc/${VERSION}/${JUJU_RELATION_ID}.relation
}

set_relation_id () {
  # Set the REMOTE_SERVICE variable to the name of the relation in the file.
  REMOTE_SERVICE=`< /etc/${VERSION}/${JUJU_RELATION_ID}.relation`
}