# This file contains functions for memcached in the tomcat charm.

add_jndi_memcached_server () {
  juju-log "Adding memcached server ${JUJU_REMOTE_UNIT} to jndi configuration"
  add_file_row "/etc/${VERSION}/${REMOTE_SERVICE}.jndi_mem" \
   "${JUJU_REMOTE_UNIT}" `relation-get host`:`relation-get port`
}

check_memcached () {
  if [ -z "`relation-get host`" ]; then
    juju-log "Memcached server ${JUJU_REMOTE_UNIT} not ready"
    exit 0
  fi
}

configure_jndi_memcached () {
  # Get the path of the .d directory where the server.xml file is built.
  server_dir=`dotdee -d /etc/${VERSION}/server.xml`
  rm -f "${server_dir}"/12-memcached-*
  # Get the path of the .d directory where the context.xml file is built.
  context_dir=`dotdee -d /etc/${VERSION}/context.xml`
  rm -f "${context_dir}"/15-memcached-*
  for name in "${!jndi_memcached_service[@]}"; do
    service=${jndi_memcached_service[${name}]}
    if [ -n "${service}" -a -e "/etc/${VERSION}/${service}.jndi_mem" ]; then
      jndi_name=`escape_name ${name}`
      (umask 0027; config_environment "${name}" "`format_memcached_servers "/etc/${VERSION}/${service}.jndi_mem"`" java.lang.String > "${server_dir}/12-memcached-${jndi_name}")
      config_resource_link "${name}" java.lang.String \
        > "${context_dir}/15-memcached-${jndi_name}"
    fi
  done
  # Build the target file /etc/dotdee/etc/tomcatX/server.xml from server.xml.d
  dotdee -u /etc/${VERSION}/server.xml
  # Build the target file /etc/dotdee/etc/tomcatX/context.xml from context.xml.d
  dotdee -u /etc/${VERSION}/context.xml
}

parse_jndi_memcached_config () {
  unset jndi_memcached_service
  declare -Ag jndi_memcached_service
  # Split the jndi_memcached_config string on commas.
  IFS="," read -a tokens <<<"`config-get jndi_memcached_config`"
  for token in "${tokens[@]}"; do
    # Split the token string using colon.
    IFS=":" read name service other <<<"${token}"
    if [ -n "${name}" ] && [ -n "${service}" ]; then
      # Put the name and service in the array if they are not empty strings.
      jndi_memcached_service[${name}]=${service}
    fi
  done
}

format_memcached_servers () {
  # format the servers in a space delimited output.
  cut -f 2 "$1" | cut -d $'\n' -f 1- --output-delimiter=" "
}

remove_jndi_memcached () {
  juju-log "Removing memcached jndi configuration for ${REMOTE_SERVICE}"
  rm -f "/etc/${VERSION}/${REMOTE_SERVICE}.jndi_mem"
}

remove_jndi_memcached_server () {
  juju-log "Removing memcached server ${JUJU_REMOTE_UNIT} from jndi configuration"
  remove_file_row "/etc/${VERSION}/${REMOTE_SERVICE}.jndi_mem" "${JUJU_REMOTE_UNIT}"
}
