# This file contains tomcat functions for the tomcat hooks.

java_debug_port=8000
jmx_port_1=10001
jmx_port_2=10002
static_cluster_port=4000

add_server () {
  remote_address=`relation-get private-address`
  unique_id=`unique_id`
  # $JUJU_UNIT_NAME variable holds the name of the unit.
  add_file_row /etc/${VERSION}/cluster.members "${JUJU_REMOTE_UNIT}" ${remote_address} ${unique_id}
  # Get the path of the .d directory where the server.xml file is built.
  dir=`dotdee -d /etc/${VERSION}/server.xml`
  if [ -e "${dir}/34-static-interceptor" ]; then
    (umask 0027; config_static_member ${remote_address} ${unique_id} > "${dir}/35-`escape_unit_name "${JUJU_REMOTE_UNIT}"`")
    dotdee -u /etc/${VERSION}/server.xml
    restart_required=true
  fi
}

check_website () {
  # If the hostname for JUJU_UNIT_NAME is not available, exit without error.
  if [ -n "`relation-get hostname "${JUJU_UNIT_NAME}"`" ]; then
    exit 0
  fi
}

close_configured_ports () {
  # For each file that matches directory/prefix*
  for port in "$1/$2"*; do
    [ -e "${port}" ] || continue
    p=${port#$1/$2}
    close_port ${p}
    # Delete the port configuration file.
    rm -f "${port}"
  done
}

config_cluster () {
  # Print the strings that configure a tomcat cluser.
  echo -n '      <Cluster className="org.apache.catalina.ha.tcp.SimpleTcpCluster" channelSendOptions="6"'
  if [ -z "$1" ]; then
    echo -n ' channelStartOptions="3"'
  fi
  echo ">"
}

config_clustering () {
  (
    umask 0027
    cat <<"EOF" > "$1/31-manager"
        <Manager className="org.apache.catalina.ha.session.DeltaManager" expireSessionsOnShutdown="false" notifyListenersOnReplication="true"/>
        <Channel className="org.apache.catalina.tribes.group.GroupChannel">
EOF
    cat <<EOF > "$1/33-cluster"
          <Receiver className="org.apache.catalina.tribes.transport.nio.NioReceiver" address="$2" port="${static_cluster_port}" autoBind="0" selectorTimeout="5000" maxThreads="6"/>
          <Sender className="org.apache.catalina.tribes.transport.ReplicationTransmitter">
            <Transport className="org.apache.catalina.tribes.transport.nio.PooledParallelSender"/>
          </Sender>
          <Interceptor className="org.apache.catalina.tribes.group.interceptors.TcpFailureDetector"/>
EOF
    cat <<"EOF" > "$1/37-cluster"
          <Interceptor className="org.apache.catalina.tribes.group.interceptors.MessageDispatch15Interceptor"/>
        </Channel>
        <Valve className="org.apache.catalina.ha.tcp.ReplicationValve" filter=".*\.gif|.*\.js|.*\.jpeg|.*\.jpg|.*\.png|.*\.htm|.*\.html|.*\.css|.*\.txt"/>
        <Valve className="org.apache.catalina.ha.session.JvmRouteBinderValve"/>
        <ClusterListener className="org.apache.catalina.ha.session.JvmRouteSessionIDBinderListener"/>
        <ClusterListener className="org.apache.catalina.ha.session.ClusterSessionListener"/>
      </Cluster>
EOF
  )
}

config_environment () {
  # http://tomcat.apache.org/tomcat-6.0-doc/config/context.html#Environment_Entries
  # Write out named values visible to web applications as environment entry resources.
  echo "    <Environment name=\"`xmlstarlet esc "$1"`\" value=\"`xmlstarlet esc "$2"`\" type=\"$3\"/>"
}

config_http_connector () {
  echo -n "    <Connector port=\"$1\" protocol=\""
  case $2 in
    bio)
      echo -n "org.apache.coyote.http11.Http11Protocol"
      ;;
    nio)
      echo -n "org.apache.coyote.http11.Http11NioProtocol"
      ;;
  esac
  echo -n '" connectionTimeout="20000" URIEncoding="UTF-8"'
  if [ -n "$3" ]; then
    echo -n " redirectPort=\"$3\""
  fi
  if [ -n "$4" ]; then
    echo -n " SSLEnabled=\"true\" scheme=\"https\" secure=\"true\" sslProtocol=\"TLS\" keystoreFile=\"/etc/${VERSION}/keystore.p12\" keystoreType=\"pkcs12\" keystorePass=\"`xmlstarlet esc "$5"`\""
  fi
  echo "/>"
}

config_jmx_listener () {
  # https://tomcat.apache.org/tomcat-7.0-doc/config/listeners.html
  echo "  <Listener className=\"org.apache.catalina.mbeans.JmxRemoteLifecycleListener\" rmiRegistryPortPlatform=\"${jmx_port_1}\" rmiServerPortPlatform=\"${jmx_port_2}\"/>"
}

config_jmx_passwords () {
  cat <<EOF
monitorRole $1
controlRole $2
EOF
}

config_multicast_membership () {
  echo "          <Membership className=\"org.apache.catalina.tribes.membership.McastService\" address=\"$1\" port=\"$2\" frequency=\"500\" dropTime=\"3000\"/>"
}

config_resource_link () {
  # http://tomcat.apache.org/tomcat-6.0-doc/config/context.html#Resource%20Links
  local name=`xmlstarlet esc "$1"`
  local type_=`xmlstarlet esc "$2"`
  # Write out the tag that creates a link to a global JNDI resource.
  echo "  <ResourceLink name=\"${name}\" global=\"${name}\" type=\"$2\"/>"
}

config_static_member () {
  # Print the strings that configure tomcat as a static member.
  echo "            <Member className=\"org.apache.catalina.tribes.membership.StaticMember\" port=\"${static_cluster_port}\" host=\"$1\" uniqueId=\"$2\"/>"
}

config_static_membership () {
  # Print the strings that configure the static membership.
  (umask 0027; echo '          <Interceptor className="org.apache.catalina.tribes.group.interceptors.StaticMembershipInterceptor">' > "$1/34-static-interceptor")
  (umask 0027; echo "          </Interceptor>" > "$1/36-static-interceptor")
}

config_users () {
  cat <<"EOF"
<?xml version="1.0" encoding="UTF-8"?>
<tomcat-users>
  <role rolename="admin-gui"/>
  <role rolename="manager-gui"/>
  <role rolename="manager-status"/>
EOF
  if [ $1 = True ]; then
    if [ -n "$2" ] && [ -n "$3" ]; then
      echo "  <user username=\"`xmlstarlet esc "$2"`\" password=\"`xmlstarlet esc "$3"`\" roles=\"admin-gui,manager-gui\"/>"
    fi
    if [ -n "$4" ] && [ -n "$5" ]; then
      echo "  <user username=\"`xmlstarlet esc "$4"`\" password=\"`xmlstarlet esc "$5"`\" roles=\"manager-status\"/>"
    fi
  fi
  echo "</tomcat-users>"
}

configure_clustering () {
  local multicast=`config-get multicast`
  local mcast_port=`config-get multicast_port`
  local mcast_address=`config-get multicast_address`
  if [ `config-get cluster_enabled` = True ]; then
    if [ ! -e "$1/31-manager" ]; then
      # Note the clustering uses the private address.
      config_clustering "$1" `unit-get private-address`
    fi
    # If clustering ever used the public address, open the port here.
    if [ ${multicast} = True ]; then
      # Multicast is enabled.
      if [ ! -e "$1/31-cluster-multicast" ]; then
        # Remove the 31-cluster-static file and all files 34-* through 36-*.
        rm -f "$1/31-cluster-static" "$1"/3[4-6]-*
        (umask 0027; config_cluster true > "$1/31-cluster-multicast")
      fi
      (umask 0027; config_multicast_membership ${mcast_address} ${mcast_port} > "$1/32-multicast")
    else
      # Multicast is not enabled.
      close_port ${mcast_port}
      if [ ! -e "$1/31-cluster-static" ]; then
        # Remove the 31-cluster-multicast file and 32-multicast file.
        rm -f "$1/31-cluster-multicast" "$1/32-multicast"
        (umask 0027; config_cluster "" > "$1/31-cluster-static")
        config_static_membership "$1"
        if [ -e /etc/${VERSION}/cluster.members ]; then
          while IFS=$'\t' read unit address unique_id; do
            (umask 0027; config_static_member ${address} ${unique_id} > "$1/35-`escape_unit_name "${unit}"`")
          done < /etc/${VERSION}/cluster.members
        fi
      fi
    fi
  else
    # Clustering is not enabled.
    # If clustering used the public address, close ports here.
    # Clustering is not enabled, remove files 31-* through 37-*
    rm -f "$1"/3[1-7]-*
  fi
}

configure_connectors () {
  # $1 =  The server directory  (example: /etc/dotdee/tomcat7/server.xml.d)
  # $2 =  The defaults directory (example: /etc/dotdee/default/tomcat7.d)
  http_port=`config-get http_port`
  http_connector=`config-get http_connector`
  https_enabled=`config-get https_enabled`
  https_port=`config-get https_port`
  # If there is not configuration file with the current http port number.
  if [ ! -e "$1/24-http-${http_port}" ]; then
    # Close the old port configuration file and delete it.
    close_configured_ports "$1" 24-http-
    open_port ${http_port}
  fi
  # Write out the http port configuration file named 24-http-<port number> 
  (umask 0027; config_http_connector ${http_port} ${http_connector} ${https_port} "" "" > "$1/24-http-${http_port}")
  if [ ${https_enabled} = True ]; then
    # If there is not a configuration file with the current https port number.
    if [ ! -e "$1/25-https-${https_port}" ]; then
      # Close the old port configuration file and delete it.
      close_configured_ports "$1" 25-https-
      open_port ${https_port}
    fi
    if [ ! -e /etc/${VERSION}/keystore.generated ]; then
      # The keystore generated does not exist, get the password from configuration.
      keystore_password=`config-get keystore_password`
    else
      # The keystore generated file exists, get the password from the file.
      keystore_password=`< /etc/${VERSION}/keystore.generated`
    fi
    (umask 0027; config_http_connector ${https_port} ${http_connector} "" true "${keystore_password}" > "$1/25-https-${https_port}")
  else
    close_configured_ports "$1" 25-https-
  fi
  if [ \( ${http_port} -le 1023 \) -o \( \( ${https_enabled} = True \) -a \( ${https_port} -le 1023 \) \) ]; then
    if [ ! -e "$2/54-authbind" ]; then
      echo "AUTHBIND=yes" > "$2/54-authbind"
    fi
  else
    rm -f "$2/54-authbind"
  fi
}

configure_java () {
  # Configure the java options.
  # Java Management Extension (JMX)
  # https://tomcat.apache.org/tomcat-7.0-doc/monitoring.html
  jmx_opts=""
  if [ `config-get jmx_enabled` = True ]; then
    if [ ! -e "$1/04-jmx-listener" ]; then
      (umask 0027; config_jmx_listener > "$1/04-jmx-listener")
    fi
    jmx_opts="-Dcom.sun.management.jmxremote.ssl=false"
    # The jmxremote.access file contains the user names.
    jmx_opts+=" -Dcom.sun.management.jmxremote.access.file=/etc/${VERSION}/jmxremote.access"
    # The jmxremote.password file contains the passwords.
    jmx_opts+=" -Dcom.sun.management.jmxremote.password.file=/etc/${VERSION}/jmxremote.password"
    if [ `config-get jmx_localhost` = True ]; then
      jmx_opts+=" -Djava.rmi.server.hostname=localhost"
    else
      # Note that JMX uses the private address of the unit.
      jmx_opts+=" -Djava.rmi.server.hostname=`unit-get private-address`"
    fi
    # If JMX used the public-address, open ports here.
  else
    # JMX is not enabled.
    # If JMX used the public-address, close ports here.
    # Remove the 04-jmx-listener file
    rm -f "$1/04-jmx-listener"
  fi
  # Java debug options.
  debug_opts=""
  if [ `config-get debug_enabled` = True ]; then 
    # Java 5.0 debug: -Xdebug -Xrunjdwp:transport=dt_socket,address=8000,server=y,suspend=n
    # From 5.0 onwards the -agentlib:jdwp option is used to load and specify options.
    debug_opts="-agentlib:jdwp=transport=dt_socket,server=y,address=${java_debug_port},suspend=n"
    open_port ${java_debug_port}
  else
    close_port ${java_debug_port}
  fi
  # Write all these Java values to the /etc/dotdee/etc/default/tomcatX.d/60-javaopts file.
  echo -n 'JAVA_OPTS="-Djava.awt.headless=true' ${debug_opts} ${jmx_opts} `config-get java_opts` \
    > "$2/60-javaopts"; echo '"' >> "$2/60-javaopts"
}

configure_keystore () {
  juju-log "Configuring keystore "
  keystore=`config-get keystore`
  if [ -n "${keystore}" ]; then
    (umask 0027; base64 -d <<<"${keystore}" > /etc/${VERSION}/keystore.p12)
    chown :${VERSION} /etc/${VERSION}/keystore.p12
    rm -f /etc/${VERSION}/keystore.generated
  else
    if [ ! -e /etc/${VERSION}/keystore.generated ]; then
      generate_keystore
    fi
  fi
}

configure_tomcat () {
  juju-log "Configuring ${VERSION}"
  configure_keystore
  server_dir=`dotdee -d /etc/${VERSION}/server.xml`
  defaults_dir=`dotdee -d /etc/default/${VERSION}`
  configure_connectors "${server_dir}" "${defaults_dir}"
  configure_clustering "${server_dir}"
  configure_java "${server_dir}" "${defaults_dir}"
  dotdee -u /etc/${VERSION}/server.xml
  dotdee -u /etc/default/${VERSION}
  config_jmx_passwords "`config-get jmx_monitor_password`" "`config-get jmx_control_password`" \
    > /etc/${VERSION}/jmxremote.password
  manager_enabled=`config-get manager_enabled`
  config_users ${manager_enabled} "`config-get admin_username`" \
    "`config-get admin_password`" "`config-get status_username`" \
    "`config-get status_password`" > /etc/${VERSION}/tomcat-users.xml
  if [ ${manager_enabled} = True ]; then
    install_tomcat_admin
  else
    remove_tomcat_admin
  fi
  configure_jndi_memcached
  configure_jndi_terracotta
}

configure_website () {
  # Called when a website relation is created.
  juju-log "Configuring website"
  relation-set hostname=`unit-get private-address` port=`config-get http_port` https-port=`config-get https_port`
}

generate_keystore () {
  juju-log "Generating self-signed certificate..."
  (umask 0077; pwgen -s 16 > /etc/${VERSION}/keystore.generated)
  (umask 0027; openssl req -x509 -subj "/CN=`escape_unit_name ${JUJU_UNIT_NAME}`" -days 1825 -newkey rsa:2048 -nodes -keyout /dev/stdout 2> /dev/null | openssl pkcs12 -export -out /etc/${VERSION}/keystore.p12 -passout fd:3 3< /etc/${VERSION}/keystore.generated 2> /dev/null)
  chown :${VERSION} /etc/${VERSION}/keystore.p12
}

install_jmx () {
  local jmx_file="catalina-jmx-remote.jar"
  if [ -f ${CHARM_DIR}/files/${jmx_file} ]; then
    juju-log "Installing ${jmx_file} from ${CHARM_DIR}/files/"
    cp "${CHARM_DIR}/files/${jmx_file}" /usr/share/${VERSION}/lib
  else
    local url=`config-get jmx_jar_url`
    local md5=`config-get jmx_jar_md5`
    juju-log "Downloading catalina jmx jar..."
    # Download using options No Verbose, Continue, to the directory Prefix temp.
    wget -nv -c -P "${TEMP}" ${url}
    dist=`basename ${url}`
    check_checksum "${TEMP}/${dist}" ${md5}

    juju-log "Installing catalina jmx jar file..."
    cp "${TEMP}/${dist}" /usr/share/${VERSION}/lib
  fi
}

install_tomcat () {
  juju-log "Installing ${VERSION} packages..."
  packages=${VERSION}
  if [ ${VERSION} = tomcat6 ]; then
    packages+=" tomcat6-extras"
  fi
  apt-get update 
  # Use the debian package manager to install the required packages.
  apt-get -y install ${packages} wget dotdee xmlstarlet pwgen dnsutils
  if [ ${VERSION} != tomcat6 ]; then
    install_jmx
  fi
  # Stop the tomcat service while it is configured.
  service ${VERSION} stop 2> /dev/null || true
  # Stop the dotdee service, it will be managed manually.
  service dotdee stop 2> /dev/null || true
  cp /dev/null /etc/init/dotdee.conf
  # Remove any dotdee server.xml configuration.
  remove_config /etc/${VERSION}/server.xml
  cp /dev/null /etc/${VERSION}/server.xml
  chmod o-r /etc/${VERSION}/server.xml
  chown :${VERSION} /etc/${VERSION}/server.xml
  # Create /etc/dotdee/etc/tomcatX/server.xml.d directory for server config.
  dotdee -s /etc/${VERSION}/server.xml
  # Get the path of the .d directory where the server.xml file is built.
  dir=`dotdee -d /etc/${VERSION}/server.xml`
  (umask 0027; cp files/server.xml.d/[0-9][0-9]* "${dir}"; cp files/server.xml.d/${VERSION}/* "${dir}")
  # Build the target file /etc/dotdee/etc/tomcatX/server.xml from server.xml.d
  dotdee -u /etc/${VERSION}/server.xml

  # Remove any dotdee context.xml configuration.
  remove_config /etc/${VERSION}/context.xml
  cp /dev/null /etc/${VERSION}/context.xml
  # Create /etc/dotdee/etc/tomcatX/content.xml.d directory for content config.
  dotdee -s /etc/${VERSION}/context.xml
  # Get the path of the .d directory where the context.xml file is built.
  dir=`dotdee -d /etc/${VERSION}/context.xml`
  cp files/context.xml.d/* "${dir}"
  # Build the target file /etc/dotdee/etc/tomcatX/context.xml from context.xml.d
  dotdee -u /etc/${VERSION}/context.xml

  # Remove any dotdee default/tomcatX configuration.
  remove_config /etc/default/${VERSION}
  # Create /etc/dotdee/etc/default/tomcatX.d directory for the default file.
  dotdee -s /etc/default/${VERSION}

  cp files/jmxremote.access /etc/${VERSION}
  (umask 0027; cp /dev/null /etc/${VERSION}/jmxremote.password)
  chown :${VERSION} /etc/${VERSION}/jmxremote.password
  rm -rf /var/lib/${VERSION}/webapps/*
  rm -f /etc/${VERSION}/Catalina/localhost/*
}

install_tomcat_admin () {
  # If tomcatX-admin package is not installed, install that package.
  if ! dpkg -s ${VERSION}-admin &> /dev/null; then
    juju-log "Installing tomcat admin..."
    apt-get -y install ${VERSION}-admin
  fi
}

obtain_tomcat_lock () {
  # Use exec and flock to lock file descriptor 9 for the tomcat install.
  if [ -e /etc/${VERSION} ]; then
    (umask 0077; touch /etc/${VERSION}/tomcat.lock)
    exec 9> /etc/${VERSION}/tomcat.lock; flock 9
  fi
}

release_tomcat_lock () {
  # Close the write file descriptor 9 used for locking.
  exec 9>&-
}

remove_server () {
  remove_file_row /etc/${VERSION}/cluster.members "${JUJU_REMOTE_UNIT}"
  # Get the path of the .d directory where the server.xml file is built.
  dir=`dotdee -d /etc/${VERSION}/server.xml`
  if [ -e "${dir}/34-static-interceptor" ]; then
    rm -f "${dir}/35-`escape_unit_name "${JUJU_REMOTE_UNIT}"`"
    # Build the target file /etc/dotdee/etc/tomcatX/server.xml from server.xml.d
    dotdee -u /etc/${VERSION}/server.xml
    restart_required=true
  fi
}

remove_tomcat_admin () {
  # If tomcatX-admin package is installed, purge that package.
  if dpkg -s ${VERSION}-admin &> /dev/null; then
    juju-log "Removing tomcat admin..."
    apt-get remove ${VERSION}-admin
  fi
}

restart_tomcat () {
  # Restart the tomcatX service.
  if service ${VERSION} status > /dev/null; then
    juju-log "Restarting ${VERSION}"
    service ${VERSION} restart
  fi
}

signed_decimal () {
  if [ $1 -gt 255 ] || [ $1 -lt -128 ]; then
    echo -128
  elif [ $1 -gt 127 ]; then
    echo $((-129 + ($1 - 127)))
  else
    echo $1
  fi
}

start_tomcat () {
  # If the tomcat service is not running start it.
  if ! service ${VERSION} status > /dev/null; then
    juju-log "Starting ${VERSION}"
    service ${VERSION} start
  fi
}

stop_tomcat () {
  # If the tomcat service is running stop it.
  if service ${VERSION} status > /dev/null; then
    juju-log "Stopping ${VERSION}"
    service ${VERSION} stop
  fi
}

unique_id () {
  # Print out a unique identifier for cluster uses.
  echo -n "{0,0,0,0,0,0,0,0,0,0,0,0"
  address=`relation-get private-address`
  # Use the domain information groper to look up the address.
  address=`dig ${address} +short`
  for o in ${address//\./ }; do
    echo -n ,`signed_decimal ${o}`
  done
  echo "}"
}
