#!/bin/bash

# This is the main hook for the tomcat charm.

set -e
set -x 

# Source the scripts with the functions that do the work.
. $CHARM_DIR/hooks/common.sh
. $CHARM_DIR/hooks/tomcat-common.sh
. $CHARM_DIR/hooks/memcached.sh
. $CHARM_DIR/hooks/terracotta.sh

# Get the tomcat version from the juju configuration.
VERSION=`config-get tomcat_version`
if [ \( "${VERSION}" != "tomcat6" \) -a \( "${VERSION}" != "tomcat7" \) ]; then
  juju-log --log-level CRITICAL "Tomcat version should be one of: tomcat6 or tomcat7"
  exit 1
fi

# Define the TEMP directory for the charm.
TEMP=/var/tmp

# Grab a tomcat lock so other charms do not install while tomcat work is done.
obtain_tomcat_lock
COMMAND=`basename $0`
case "$COMMAND" in
  install)
    install_tomcat
    ;;
  start)
    start_tomcat
    ;;
  stop)
    stop_tomcat
    ;;
  config-changed)
    parse_jndi_memcached_config
    parse_jndi_terracotta_config
    configure_tomcat
    restart_tomcat
    ;;
  upgrade-charm)
    parse_jndi_memcached_config
    parse_jndi_terracotta_config
    configure_tomcat
    restart_tomcat
    ;;
  cluster-relation-departed)
    remove_server
    if [ -n "${restart_required}" ]; then
      restart_tomcat
    fi
    ;;
  cluster-relation-joined)
    add_server
    if [ -n "${restart_required}" ]; then
      restart_tomcat
    fi
    ;;
  website-relation-joined)
    check_website
    configure_website
    ;;
  jndi-memcached-relation-broken)
    set_relation_id
    remove_jndi_memcached
    parse_jndi_memcached_config
    configure_jndi_memcached
    restart_tomcat
    remove_relation_id
    ;;
  jndi-memcached-relation-changed)
    check_memcached
    add_jndi_memcached_server
    parse_jndi_memcached_config
    configure_jndi_memcached
    restart_tomcat
    ;;
  jndi-memcached-relation-departed)
    remove_jndi_memcached_server
    parse_jndi_memcached_config
    configure_jndi_memcached
    restart_tomcat
    ;;
  jndi-memcached-relation-joined)
    create_relation_id
    ;;
  jndi-terracotta-relation-broken)
    set_relation_id
    remove_jndi_terracotta
    parse_jndi_terracotta_config
    configure_jndi_terracotta
    restart_tomcat
    remove_relation_id
    ;;
  jndi-terracotta-relation-changed)
    check_terracotta
    add_jndi_terracotta_server
    parse_jndi_terracotta_config
    configure_jndi_terracotta
    restart_tomcat
    ;;
  jndi-terracotta-relation-departed)
    remove_jndi_terracotta_server
    parse_jndi_terracotta_config
    configure_jndi_terracotta
    restart_tomcat
    ;;
  jndi-terracotta-relation-joined)
    create_relation_id
    create_terracotta_log_directory
    ;;
  *)
    juju-log "Unknown command $COMMAND"
    ;;
esac
# Release the tomcat lock.
release_tomcat_lock
