#!/usr/bin/python3

# This amulet code is to test the tomcat charm.

import amulet
import base64
import os
import requests
import subprocess

# The number of seconds to wait for Juju to set up the environment.
seconds = 1200
# The number of units to scale to.
scale = 2
# Get the directory in this way to load the files from the tests directory.
path = os.path.abspath(os.path.dirname(__file__))
# Get the path to the keystore file.
keystore_path = os.path.join(path, 'keystore.p12')
# Take the sha1sum of the keystore file.
command = ['sha1sum', keystore_path]
process = subprocess.Popen(command, stdout=subprocess.PIPE)
output = process.stdout.read()
# Decode and strip off the new line.
keystore_sha1sum = output.decode().strip()
print(keystore_sha1sum)
# Read in the keystore file.
with open(keystore_path, 'rb') as f:
    keystore = f.read()
# base64 encode the keystore bytes.
base64_encoded_keystore = base64.b64encode(keystore)
# The configuration to send to the tomcat charm.
tomcat_configuration = {
    'tomcat_version': 'tomcat7',
    'http_port': 80,
    'http_connector': 'nio',
    'https_enabled': True,
    'https_port': 443,
    'keystore': base64_encoded_keystore.decode('ascii'),
    'keystore_password': 'keystore_p@ssw0rd',
    'manager_enabled': True,
    'admin_username': 'admin',
    'admin_password': 'admin_p@ssw0rd',
    'status_username': 'status',
    'status_password': 'status_p@ssw0rd',
    'java_opts': '-Xms768M -Xmx1024M',
    'debug_enabled': True,
    'jmx_enabled': True,
    'jmx_localhost': True,
    'jmx_control_password': 'jmx_control_p@ssw0rd',
    'jmx_monitor_password': 'jmx_monitor_p@ssw0rd',
    'jmx_jar_url': 'http://archive.apache.org/dist/tomcat/tomcat-7/v7.0.26/bin/extras/catalina-jmx-remote.jar',
    'jmx_jar_md5': '82763603dc177d9aabd17d39cef7dcbb',
    'jndi_memcached_config': 'param/Memcached:relation-sentry',
    'jndi_terracotta_config': 'param/Terracotta:relation-sentry',
    'cluster_enabled': True
}

d = amulet.Deployment(series='trusty')
# Add the tomcat charm to the deployment.
d.add('tomcat', units=scale)
d.add('memcached')
d.add('terracotta')

# Configure the tomcat charm.
d.configure('tomcat', tomcat_configuration)

# Relate the tomcat charm.
d.relate('tomcat:jndi-memcached', 'memcached:cache')
d.relate('tomcat:jndi-terracotta', 'terracotta:dso')

# Expose the open ports on the tomcat charm.
d.expose('tomcat')

# Deploy the environment and wait for it to setup.
try:
    d.setup(timeout=seconds)
    #d.sentry.wait(seconds)
except amulet.helpers.TimeoutError:
    message = 'The environment did not setup in %d seconds.' % seconds
    # The SKIP status enables skip or fail the test based on configuration.
    amulet.raise_status(amulet.SKIP, msg=message)
except:
    raise

# Get the sentry unit for tomcat0.
tomcat_unit = d.sentry['tomcat'][0]
# Get the public address for tomcat0.
tomcat0_address = tomcat_unit.info['public-address']
# Get the terracotta unit.
terracotta_unit = d.sentry['terracotta'][0]
# Get the relation to terracotta.
terracotta_relation = terracotta_unit.relation('dso', 'tomcat:jndi-terracotta')
# Get the private address of terracotta to ensure the relation has that data.
terracotta_private_address = terracotta_relation['private-address']
print("terracotta private address " + terracotta_private_address)
# Get the memcached unit.
memcached_unit = d.sentry['memcached'][0]
# Get the relation to memcached.
memcached_relation = memcached_unit.relation('cache', 'tomcat:jndi-memcached')
# Get the private address of memcached to ensure the relation has that data.
memcached_private_address = memcached_relation['private-address']
print("memcached private address " + memcached_private_address)

# Save some of the often used configuration options to variables for reuse.
http_port = tomcat_configuration['http_port']
https_port = tomcat_configuration['https_port']
tomcat_version = tomcat_configuration['tomcat_version']

# Get the private-address from the tomcat/1 instance.
# The following code is really ugly, but it necessary because the last
# argument needs to be quoted and Popen does a good job at escaping quotes.
command = 'juju run --unit tomcat/1 "unit-get private-address"'
process = subprocess.Popen(command, stdout=subprocess.PIPE, shell=True)
# Get the stdout from communicate method.
address = process.communicate()[0]
print(address)
# Convert from bytes to string and strip the new line.
tomcat1_address = address.decode().rstrip('\n')

###############################################################################
## Read the /etc/tomcatX/default file to verify configuration settings.
###############################################################################
# Read in the default file from the tomcat unit.
default_tomcat = tomcat_unit.file_contents('/etc/default/{0}'.format(
                                           tomcat_version))
# This call will fail with an IO exception if the file does not exist.

# Search for java_opts in the default file and raise exception if not found.
default_tomcat.index(tomcat_configuration['java_opts'])
# Search for debug flag in the default file and raise exception if not found.
default_tomcat.index('agentlib:jdwp=transport')

# Search for JMX access in the default file and raise exception if not found.
default_tomcat.index('jmxremote.access.file=')
# Search for JMX password in the default file and raise exception if not found.
default_tomcat.index('jmxremote.password.file=')
# Search for JMX local in the default file and raise exception if not found.
default_tomcat.index('java.rmi.server.hostname=localhost')

###############################################################################
## Verify the JMX settings.
###############################################################################
# Read in the jmx password file from the tomcat unit.
jmx_passwords = tomcat_unit.file_contents('/etc/{0}/jmxremote.password'.format(
                                          tomcat_version))
# This method would fail with an IO Exception if the file does not exist.

# Search for control in the JMX password file and raise exception if not found.
jmx_passwords.index(tomcat_configuration['jmx_control_password'])
# Search for monitor in the JMX password file and raise exception if not found.
jmx_passwords.index(tomcat_configuration['jmx_monitor_password'])

###############################################################################
## Read in the /etc/tomcatX/server.xml file to verify configuration settings.
###############################################################################
# Get the server.xml file based on the tomcat version.
server_xml = tomcat_unit.file_contents('/etc/{0}/server.xml'.format(
                                       tomcat_version))
# This call would fail with an IO Exception if the file does not exist.

# Search for http port in the server file raise an exception if not found.
server_xml.index('Connector port="{0}"'.format(http_port))
# Search for https port in the server file raise an exception if not found.
server_xml.index('Connector port="{0}"'.format(https_port))

# Search for the StaticMember class.
index = server_xml.index("org.apache.catalina.tribes.membership.StaticMember")
# Grab the entire line.
static_member = server_xml[index:].split('\n')[0]
print(static_member)
print(tomcat1_address)
# Search for tomcat1 address in the tomcat0 server.xml to verify clustered.
static_member.index(tomcat1_address)

###############################################################################
## Verify the keystore has the same hash value.
###############################################################################
command = 'sha1sum /etc/{0}/keystore.p12'.format(tomcat_version)
remote_sha1sum, code = tomcat_unit.run(command)
print(remote_sha1sum)
# Check if the remote checksum matches the original.
if remote_sha1sum.split()[0] != keystore_sha1sum.split()[0]:
    message = "The keystore.p12 sha1sum did not match the remote server!"
    amulet.raise_status(amulet.FAIL, msg=message)

# Get the content.xml file based on the tomcat version.
context_xml = tomcat_unit.file_contents('/etc/{0}/context.xml'.format(
                                        tomcat_version))
# This call would fail with an IO Exception if the file does not exist.

###############################################################################
## Verify the tomcat status page is available and status can authenticate.
###############################################################################
# Create a url for the tomcat status page.
tomcat_status_url = 'http://{0}:{1}/manager/status'.format(tomcat0_address,
                                                           http_port)
# Create a status authentication tuple to authenticate to the status page.
status_auth = (tomcat_configuration['status_username'],
               tomcat_configuration['status_password'])
print(tomcat_status_url)
# Issue a GET request for the status url with configured authentication.
response = requests.get(tomcat_status_url, auth=status_auth)
response.raise_for_status()
print('Successfully authenticated to the tomcat status url: %s' %
      tomcat_status_url)

# Search for the http connector in the status response text.
response.text.index('-{0}-'.format(tomcat_configuration['http_connector']))
# Raise an exception if the http_connector value is not in the status string.

############################################################################
## Verify the tomcat manager page is avialable and admin can authenticate.
############################################################################
# Create a url for the tomcat server's management plugin.
tomcat_manager_url = 'http://{0}:{1}/manager/html'.format(tomcat0_address,
                                                          http_port)
print(tomcat_manager_url)
# Create the authentication tuple to authenticate to the manager/html page.
authentication = (tomcat_configuration['admin_username'],
                  tomcat_configuration['admin_password'])
# Issue a GET request for the management url with configured authentication.
response = requests.get(tomcat_manager_url, auth=authentication)
response.raise_for_status()
print('Successfully authenticated to the tomcat manager url:  %s' %
      tomcat_manager_url)
# The manager response is used in the next few steps.

###############################################################################
## Deploy the TestWebApplication to verify the Servlet and JSP functionality.
###############################################################################
# Find the index of the deploy string in the response.
start = response.text.find('/manager/html/upload;')
# Find the index of the next double quote in the the response.
stop = response.text.find('"', start)
# Create the deploy url from these values.
deploy_url = 'http://{0}:{1}{2}'.format(tomcat0_address, http_port,
                                        response.text[start:stop])
print(deploy_url)
# Create the path for the TestApplication WAR file.
war_file = os.path.join(path, 'TestWebApplication.war')
# Create the POST multipart encoded file object.
files = {'deployWar': ('TestWebApplication.war', open(war_file, 'rb'),
         'application/binary')}
# Issue the post request that will deploy the servlet.
response = requests.post(deploy_url, auth=authentication, files=files)
response.raise_for_status()

# Create the url to access the deployed web application.
webapp_url = 'http://{0}:{1}/TestWebApplication'.format(tomcat0_address,
             http_port)
print(webapp_url)
# Issue a get request to make sure the web app is deployed.
response = requests.get(webapp_url)
response.raise_for_status()
print("TestWebApplication successfully deployed on: " + tomcat0_address)

###############################################################################
## Verify that tomcat can run Java Server Pages (JSP).
###############################################################################
jsp_url = webapp_url + "/test.jsp"
print(jsp_url)
# Get the jsp_url to search for JSP strings.
response = requests.get(jsp_url)
response.raise_for_status()
index = response.text.find("If you can read this String, JSP is working.")
if index == -1:
    message = "JSPs are not working on tomcat!"
    amulet.raise_status(amulet.FAIL, msg=message)
# This test has just verified the JSP functionality of tomcat.
print("Verified that tomcat can run JSPs!")

###############################################################################
## Verify that tomcat can run Servlets.
###############################################################################
# Create the TestWebApplication servlet url.
servlet_url = webapp_url + '/TestServlet'
print(servlet_url)
# Get the TestWebapplication servlet response.
response = requests.get(servlet_url)
response.raise_for_status()
# This test proved that the web application was accessible after deployment.
print("Verified that tomcat can run Servlets!")

###############################################################################
## Verify that the Java Naming and Directory Information service works.
###############################################################################
# Search for the JNDI name that is printed by the servlet.
memcached_name = 'param/Memcached='
index = response.text.find(memcached_name)
if index == -1:
    message = "Could not find %s in JNDI." % memcached_name
    amulet.raise_status(amulet.FAIL, msg=message)
# Add index to length and get the current line for the value.
memcached_value = response.text[(index + len(memcached_name)):].split()[0]
print(memcached_name + memcached_value)
print("Searching for: " + memcached_private_address)
# The JNDI value must contain the memcached address otherwise raise an error.
memcached_value.index(memcached_private_address)

# Search for the JNDI name that is printed by the servlet.
terracotta_name = 'param/Terracotta='
index = response.text.find(terracotta_name)
if index == -1:
    message = "Could not find %s in JNDI." % terracotta_name
    amulet.raise_status(amulet.FAIL, msg=message)
# Add index to length and get the current line for the value.
terracotta_value = response.text[(index + len(terracotta_name)):].split()[0]
print(terracotta_name + terracotta_value)
print("Searching for: " + terracotta_private_address)
# The JNDI value must contain the terracotta address otherwise raise an error.
terracotta_value.index(terracotta_private_address)

print("The tomcat tests are complete!")
